package wad.notebook;

import fi.helsinki.cs.tmc.edutestutils.Points;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import static org.assertj.core.api.Assertions.assertThat;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringRunner.class)
@SpringBootTest
@AutoConfigureMockMvc
@Points("01-10")
public class NotebookTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    public void contentFromFormAddedToNotebook() throws Exception {
        addContent(3, 10);
    }

    @Test
    public void onlyTenLatestMessagesShown() throws Exception {
        addContent(25, 10);
    }

    public void addContent(int num, int maxSize) throws Exception {
        List<String> items = new ArrayList<>();
        List<String> shouldNotContainItems = new ArrayList<>();

        for (int i = 0; i < num; i++) {
            String contentToAdd = "TODO: " + UUID.randomUUID().toString();

            String pageContent = this.mockMvc.perform(get("/")).andDo(print()).andExpect(status().isOk())
                    .andReturn().getResponse().getContentAsString();

            shouldNotContainItems.stream().forEach(s -> assertThat(pageContent).doesNotContain(s));

            this.mockMvc.perform(post("/").param("content", contentToAdd))
                    .andExpect(status().is3xxRedirection())
                    .andReturn();

            items.add(contentToAdd);
            while (items.size() > maxSize) {
                shouldNotContainItems.add(items.remove(0));
            }

            String pageContentPost = this.mockMvc.perform(get("/")).andDo(print()).andExpect(status().isOk())
                    .andReturn().getResponse().getContentAsString();

            items.stream().forEach(s -> assertThat(pageContentPost).contains(s));
        }
    }
}
